from get_sn_token import get_token
from create_sn_co import create_change_order
from get_sn_co_item import get_configuration_item
from risk_assessment import risk_assessment
from conflict_check import conflict_status_check, start_conflict_check
from update_status import cancel_change, move_status, change_status
from date_logic import dates_calculations

def create_schedule_co (SN_INSTANCE, sn_access_token, CO_ITEM, start_date, end_date, year_week,
                       data_center, environment_name, co_properties,
                       is_test_mode=False):

    week_days = {
        0: "Monday",
        1: "Tuesday",
        2: "Wednesday",
        3: "Thursday",
        4: "Friday",
        5: "Saturday",
        6: "Sunday",
        7: "Monday",
        8: "Tuesday",
        9: "Wednesday",
        10: "Thursday",
        11: "Friday",
        12: "Saturday",
        13: "Sunday"
    }

    # if day of the week was not passed in the job, default to Thursday.
    if days_of_week is None:
        days_of_week = [3]
    else:
        days_of_week = days_of_week.replace(" ",",")
        logging.warning(f"day_of_week:{days_of_week}")
        days_of_week = json.loads(days_of_week)

    # if this is a cron - default to a team member if not provided. For one time job, we make it mandatory from the Jenkins dashboard.
    if requested_by is None:
        requested_by = co_properties.default_requested_by;

    for day in days_of_week:
        logging.warning(f"For day :{day}")
        start_date, end_date, week_year = dates_calculations(day)
        sn_access_token = get_sn_token.get_token(SN_USER, SN_PASSWORD, SN_INSTANCE)

        if sn_access_token is not None:
            access_token = sn_access_token["result"]["access_token"]
            if access_token is not None:
                logging.warning(f"Target date {week_days[day]}, week #{week_year}, Start time: {start_date}, end time: {end_date}")
                # retrieve Configuration item environment specific
                CO_ITEM_T = get_configuration_item(SN_INSTANCE, access_token,configuration_item_name)
                configuration_item_id = (CO_ITEM_T["result"][0])["sys_id"]

                # create CO request
                co = create_sn_co.create_change_order(SN_INSTANCE, access_token, configuration_item_id, start_date, end_date, week_year, requested_by, data_center, environment_name, False)
                co_number = (co["result"][0])["Change ID"]
                current_datetime = f"{datetime.now():%m-%d-%Y %H:%M:%S}"
                logging.warning(f"{current_datetime} - {co_number} Scheduled for: {week_days[day]}, week #{week_year}, Start time: {start_date}, end time: {end_date}")

                # Risk Assessment
                co_ra = risk_assessment.risk_assessment(SN_INSTANCE, access_token, co_number)
                current_datetime = f"{datetime.now():%m-%d-%Y %H:%M:%S}"
                logging.warning(f"{current_datetime} - CO Risk Assessment: {co_ra}")

                # Start Conflict Check
                co_cc = conflict_check.start_conflict_check(SN_INSTANCE, access_token, co_number)
                current_datetime = f"{datetime.now():%m-%d-%Y %H:%M:%S}"
                logging.warning(f"{current_datetime} - CO Conflict check started to trigger conflict detection: {co_cc['result'][0]}")

                # Start waiting time (in seconds). As per SN instructions we should wait 5 minutes before calling the status check
                # Increase by 3 hours to allow operator in charge to update the ticket details
                sleep(5*60)
                current_datetime = f"{datetime.now():%m-%d-%Y %H:%M:%S}"
                logging.warning(f"{current_datetime} - Waiting time is over, checking status now")

                # Conflict Status Check
                co_csc = conflict_check.conflict_status_check(SN_INSTANCE, access_token, co_number)
                co_status = co_csc["result"]["status"]
                current_datetime = f"{datetime.now():%m-%d-%Y %H:%M:%S}"

                # Wait additional 3 hours, before updating status to asses. This will allow operator in charge to update the ticket details
                # sleep(3*60*60)
                if co_status=="Conflict":
                    logging.warning(f"{current_datetime} - Conflict identified, please review it before requesting approval")
                    # if conflict, update Change Exception Type
                    co_cs = update_status.change_status(SN_INSTANCE, access_token, co_number)
                    logging.warning(f"CO Change status result: {co_cs['result']}")
                else:
                    logging.warning(f"{current_datetime} - No conflict, proceed with move status")
                    co_cs = update_status.move_status(SN_INSTANCE, access_token, co_number)
                    logging.warning(f"CO Change status result: {co_cs['result']}")

                # to cancel a ticket
                    canceled = update_status.cancel_change(SN_INSTANCE, access_token, co_number)
                    logging.warning(f"CO Change canceled: {canceled}")


'''
Invoke conflict check by accessing SN endpoint. This will start the conflict engine.
The confirmation message is a directive to wait 5 minutes before checking the status.
'''
import requests
import logging

class ConflictCO:
    def __init__(self, url, headers):
        self.headers = headers
        self.url = url

    def retrieve_change(self):
        response = requests.post(self.url, headers=self.headers)
          
        if (response.status_code != 200 and response.status_code != 201 and response.status_code != 202):
            logging.error({'Status:': response.status_code})
            logging.error({'Full response:': response.json()})
            exit()

        # Decode the JSON response into a dictionary and use the data
        resp = response.json()
        return resp
    
    def status_check(self):
        response = requests.get(self.url, headers=self.headers)
          
        if (response.status_code != 200 and response.status_code != 201 and response.status_code != 202):
            logging.error({'Status:': response.status_code})
            logging.error({'Full response:': response.json()})
            exit()

        # Decode the JSON response into a dictionary and use the data
        resp = response.json()
        return resp

def start_conflict_check(SN_INSTANCE, access_token, co_number):
    url = f'https://{SN_INSTANCE}.service-now.com/api/audp/adp_change/change/{co_number}/conflict'
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': f'Bearer {access_token}'
        }
    co = ConflictCO(url, headers)
    sn_co = co.retrieve_change()
    return sn_co

def conflict_status_check(SN_INSTANCE, access_token, co_number):
    url = f'https://{SN_INSTANCE}.service-now.com/api/audp/adp_change/conflict_status_check/{co_number}'
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': f'Bearer {access_token}'
        }
    co = ConflictCO(url, headers)
    sn_co = co.status_check()
    return sn_co


"""
Set the risk level for a Change by using the pre-defined Risk Assessment Value (RAV) Calculator. 

The RAV is calculated using the following Risk Assessment questions:
Question 
    Weight (Value)  : Possible Answers

Outage Scope/Complexity 
    8   :   Complete service outage or high complexity involving coordinating activity of multiple services, groups, or vendors. Service down during maintenance window.
    4   :   Partial service outage / degradation or moderate complexity involving coordinating activity of multiple services, groups, or vendors. Likely to experience loss of functionality or capacity during maintenance window.
    2   :   Minor service outage / degradation or low complexity involving coordinating activity of single services or groups. Likely to experience short loss of functionality or capacity during maintenance window.
    0   :   No impact or complexity

Locations or # Business
    8   :   Units Impacted Enterprise or > 4 BUs
    2   :   Single location or 1 BU
    1   :   Single location and 0 BUs

Business Impact
    8   :   Very high impact or visibility for ADP businesses. Will impact downstream systems.
    4   :   Medium impact or visibility for affected customers. Likely to affect downstream systems.
    1   :   Low impact or visibility for affected customers. Not likely to affect downstream services.

Backout /Rollback Plan 
    8   :   Unable to back out change or requires a complete system restore if implementation fails. Will or likely to require additional downtime
    4   :   Challenging level of difficulty to backout or backout not desired due to dependencies on data and other systems regardless of level of documentation available. 
            Likely to require additional downtime outside maintenance window to complete backout plan.
    2   :   Moderate level of difficulty to backout due to dependencies on data and other system or inadequate documentation. Likely able to complete backout within the maintenance window.
    1   :   Routine level of difficulty to backout. Adequate documentation and similar backout has been performed in the past. Able to complete backout within the maintenance window.
    0   :   Fully automated rollback within maintenance window.

Experience with Change
    8   :   No experience with this type of change or unable to fully test the change.
    6   :   Limited experience or success in making this type of change or unable to fully test the change. Few previous changes of this type successfully completed.
    2   :   Moderate experience with this type of change or the ability to partially test the change. Most previous changes of this type were successfully completed.
    1   :   Extensive experience with this type of change or ability to fully test the change. Previous changes of this type successfully completed.

Certification 
    8   :   Unable to identify and verify successful operation of all affected application and configuration items.
    0   :   Able to identify and verify successful operation of all affected application and configuration items.
"""
import requests
import logging
import json

class AssessRisk:
    def __init__(self, url, headers, body):
        self.headers = headers
        self.url = url
        self.body = body

    def retrieve_change(self):
        response = requests.post(self.url, headers=self.headers, data=self.body)
          
        if (response.status_code != 200 and response.status_code != 201):
            logging.error({'Status:': response.status_code})
            logging.error({'Full response:': response.json()})
            exit()

        # Decode the JSON response into a dictionary and use the data
        resp = response.json()
        return resp

def risk_assessment(sn_instance, sn_access_token, change_number):
    url = f'https://{sn_instance}.service-now.com/api/audp/adp_change/change_risk_assessment/{change_number}'
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': f'Bearer {sn_access_token}'
        }

    body = {
        "Outage Scope/Complexity": {
            "value": "2"
        },
        "Business Impact": {
            "value": "4"
        },
        "Backout /Rollback Plan": {
            "value": "0"
        },
        "Experience with Change": {
            "value": "1"
        },
        "Locations or # Business Units Impacted": {
            "value": "8"
        },        
        "Certification": {
            "value": "0"
        }
    }
    co = AssessRisk(url, headers, json.dumps(body))
    sn_co = co.retrieve_change()
    return sn_co


import requests
import logging

class CMDBCi:
    def __init__(self, url, headers):
        self.headers = headers
        self.url = url

    def retrieve_change(self):
        response = requests.get(self.url, headers=self.headers)
          
        if (response.status_code != 200 and response.status_code != 201):
            logging.error({'Status:': response.status_code})
            logging.error({'Full response:': response.json()})
            exit()

        # Decode the JSON response into a dictionary and use the data
        resp = response.json()
        return resp

def get_configuration_item(SN_INSTANCE, sn_access_token,configuration_item):
    url = f'https://{SN_INSTANCE}.service-now.com/api/now/table/cmdb_ci?sysparm_query=name={configuration_item}&sysparm_fields=sys_id'
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': f'Bearer {sn_access_token}'
        }
    co = CMDBCi(url, headers)
    sn_co = co.retrieve_change()
    return sn_co